change_spec = {
    "paddle.fluid.layers.nn.pow": {
        "alias": [
            "paddle.fluid.layers.pow"
        ],
        "update_to": "paddle.pow",
        "args_list": [
            "x",
            "factor",
            "name"
        ],
        "args_change": [
            [
                "factor",
                "y"
            ],
        ]
    },
    "paddle.fluid.layers.ops.ceil": {
        "alias": [
            "paddle.fluid.layers.ceil"
        ],
        "update_to": "paddle.ceil"
    },
    "paddle.fluid.layers.control_flow.while_loop": {
        "alias": [
            "paddle.fluid.layers.while_loop"
        ],
        "update_to": "paddle.static.nn.while_loop"
    },
    "paddle.fluid.layers.tensor.trace": {
        "alias": [
            "paddle.fluid.layers.trace"
        ],
        "update_to": "paddle.trace",
        "args_list": [
            "input",
            "offset",
            "dim1",
            "dim2",
            "out",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim1",
                "axis1"
            ],
            [
                "dim2",
                "axis2"
            ],
            [
                "out",
                ""
            ]
        ],
        "args_warning": {
            "out": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.load": {
        "alias": [
            "paddle.fluid.io.load"
        ],
        "update_to": "paddle.static.load"
    },
    "paddle.fluid.layers.tensor.full_like": {
        "alias": [
            "paddle.fluid.layers.full_like"
        ],
        "update_to": "paddle.full_like",
        "args_list": [
            "input",
            "fill_value",
            "out",
            "dtype",
            "device",
            "stop_gradient",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "out",
                ""
            ],
            [
                "device",
                ""
            ],
            [
                "stop_gradient",
                ""
            ]
        ],
        "args_warning": {
            "out": ""
        }
    },
    "paddle.fluid.layers.tensor.argmax": {
        "alias": [
            "paddle.fluid.layers.argmax"
        ],
        "update_to": "paddle.argmax"
    },
    "paddle.fluid.dygraph.nn.PRelu": {
        "alias": [
            "paddle.fluid.dygraph.PRelu"
        ],
        "warning": "this api is update to paddle.nn.PReLU"
    },
    "paddle.fluid.layers.ops.tanh_shrink": {
        "alias": [
            "paddle.fluid.layers.tanh_shrink"
        ],
        "update_to": "paddle.nn.functional.tanhshrink"
    },
    "paddle.fluid.layers.tensor.linspace": {
        "alias": [
            "paddle.fluid.layers.linspace"
        ],
        "update_to": "paddle.linspace",
    },
    "paddle.fluid.layers.expand": {
        "alias": [
            "paddle.fluid.layers.nn.expand"
        ],
        "warning": "this api is update to paddle.expand"
    },
    "paddle.fluid.backward.append_backward": {
        "update_to": "paddle.static.append_backward"
    },
    "paddle.fluid.layers.diag_embed": {
        "alias": [
            "paddle.fluid.layers.nn.diag_embed"
        ],
        "update_to": "paddle.nn.functional.diag_embed"
    },
    "paddle.fluid.layers.square_error_cost": {
        "alias": [
            "paddle.fluid.layers.loss.square_error_cost"
        ],
        "update_to": "paddle.nn.functional.square_error_cost"
    },
    "paddle.fluid.layers.detection.multi_box_head": {
        "alias": [
            "paddle.fluid.layers.multi_box_head"
        ],
        "update_to": "paddle.static.nn.multi_box_head"
    },
    "paddle.fluid.dygraph.InverseTimeDecay": {
        "alias": [
            "paddle.fluid.dygraph.learning_rate_scheduler.InverseTimeDecay"
        ],
        "warning": "this api is update to paddle.expand"
    },
    "paddle.fluid.layers.asin": {
        "alias": [
            "paddle.fluid.layers.ops.asin"
        ],
        "update_to": "paddle.asin"
    },
    "paddle.fluid.Program": {
        "alias": [
            "paddle.fluid.framework.Program"
        ],
        "update_to": "paddle.static.Program"
    },
    "paddle.fluid.metrics.Auc": {
        "warning": "this api is update to paddle.metric.Auc"
    },
    "paddle.fluid.layers.nn.reshape": {
        "alias": [
            "paddle.fluid.layers.reshape"
        ],
        "warning": "this api is update to paddle.reshape"
    },
    "paddle.fluid.layers.control_flow.greater_than": {
        "alias": [
            "paddle.fluid.layers.greater_than"
        ],
        "update_to": "paddle.greater_than",
        "args_list": [
            "x",
            "y",
            "cond"
        ],
        "args_change": [
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "cond": "this args is deleted in this version"
        }
    },
    "paddle.fluid.compiler.CompiledProgram": {
        "alias": [
            "paddle.fluid.CompiledProgram"
        ],
        "update_to": "paddle.static.CompiledProgram"
    },
    # TODO transformer
    "paddle.fluid.layers.flatten": {
        "alias": [
            "paddle.fluid.layers.nn.flatten"
        ],
        "warning": "this api is update to paddle.flatten"
    },
    "paddle.fluid.dygraph.nn.BCELoss": {
        "alias": [
            "paddle.fluid.dygraph.BCELoss"
        ],
        "update_to": "paddle.nn.BCELoss",
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.ExponentialDecay": {
        "alias": [
            "paddle.fluid.dygraph.ExponentialDecay"
        ],
        # "update_to": "paddle.optimizer.lr.ExponentialDecay",
        "warning": "this api is update to paddle.optimizer.lr.ExponentialDecay"
    },
    "paddle.fluid.layers.detection.ssd_loss": {
        "alias": [
            "paddle.fluid.layers.ssd_loss"
        ],
        "update_to": "paddle.nn.functional.ssd_loss"
    },
    "paddle.fluid.layers.nn.logical_not": {
        "alias": [
            "paddle.fluid.layers.logical_not"
        ],
        "update_to": "paddle.logical_not"
    },
    # manual check
    "paddle.fluid.layers.nn.randint": {
        "alias": [
            "paddle.fluid.layers.randint"
        ],
        "update_to": "paddle.randint",
        "args_list": [
            "low",
            "high",
            "shape",
            "out",
            "dtype",
            "device",
            "stop_gradient",
            "seed",
            "name"
        ],
        "args_change": [
            [
                "out",
                ""
            ],
            [
                "device",
                ""
            ],
            [
                "stop_gradient",
                ""
            ],
            [
                "seed",
                ""
            ]
        ],
        "args_warning": {
            "out": "this args is deleted in this version",
            "device": "this args is deleted in this version",
            "stop_gradient": "this args is deleted in this version",
            "seed": "this args is deleted in this version"
        }
    },
    "paddle.fluid.enable_imperative": {
        "alias": [
            "paddle.fluid.dygraph.base.enable_imperative",
            "paddle.fluid.dygraph.enable_imperative"
        ],
        "update_to": "paddle.disable_static"
    },
    "paddle.fluid.layers.softplus": {
        "alias": [
            "paddle.fluid.layers.ops.softplus"
        ],
        "update_to": "paddle.nn.functional.softplus",
    },
    "paddle.fluid.layers.ops.softshrink": {
        "alias": [
            "paddle.fluid.layers.softshrink"
        ],
        "update_to": "paddle.nn.functional.softshrink",
        "args_list": [
            "x",
            "alpha"
        ],
        "args_change": [
            [
                "alpha",
                "threshold"
            ]
        ]
    },
    "paddle.fluid.dygraph.Pool2D": {
        "alias": [
            "paddle.fluid.dygraph.nn.Pool2D"
        ],
        "warning": "please use paddle.nn.AvgPool2D or paddle.nn.MaxPool2D"
    },
    "paddle.fluid.layers.mean_iou": {
        "alias": [
            "paddle.fluid.layers.nn.mean_iou"
        ],
        "update_to": "paddle.metric.mean_iou"
    },
    "paddle.fluid.layers.multiplex": {
        "alias": [
            "paddle.fluid.layers.nn.multiplex"
        ],
        "update_to": "paddle.multiplex"
    },
    "paddle.fluid.embedding": {
        "alias": [
            "paddle.fluid.input.embedding",
            "paddle.fluid.layers.embedding",
            "paddle.fluid.layers.nn.embedding"
        ],
        "update_to": "paddle.static.nn.embedding"
    },
    "paddle.fluid.io.load_inference_model": {
        "update_to": "paddle.static.load_inference_model"
    },
    "paddle.fluid.layers.nn.scale": {
        "alias": [
            "paddle.fluid.layers.scale"
        ],
        "update_to": "paddle.scale"
    },
    # manual check
    "paddle.fluid.layers.full": {
        "alias": [
            "paddle.fluid.layers.tensor.full"
        ],
        "update_to": "paddle.full",
        "args_list": [
            "shape",
            "fill_value",
            "out",
            "dtype",
            "device",
            "stop_gradient",
            "name"
        ],
        "args_change": [
            [
                "out",
                ""
            ],
            [
                "device",
                ""
            ],
            [
                "stop_gradient",
                ""
            ]
        ],
        "args_warning": {
            "out": "this args is deleted in this version",
            "device": "this args is deleted in this version",
            "stop_gradient": "this args is deleted in this version"
        }
    },
    "paddle.fluid.layers.create_parameter": {
        "alias": [
            "paddle.fluid.layers.tensor.create_parameter"
        ],
        "update_to": "paddle.create_parameter"
    },
    # manual check
    "paddle.fluid.layers.nn.softmax": {
        "alias": [
            "paddle.fluid.layers.softmax"
        ],
        "update_to": "paddle.nn.functional.softmax",
        "args_list": [
            "input",
            "use_cudnn",
            "name",
            "axis"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "use_cudnn",
                ""
            ]
        ],
        "args_warning": {
            "use_cudnn": "This args is deleted in this version"
        }
    },
    "paddle.fluid.layers.is_empty": {
        "alias": [
            "paddle.fluid.layers.control_flow.is_empty"
        ],
        "update_to": "paddle.is_empty",
        "args_list": [
            "x",
            "cond"
        ],
        "args_change": [
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "cond": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.dygraph.Conv3DTranspose": {
        "alias": [
            "paddle.fluid.dygraph.nn.Conv3DTranspose"
        ],
        "update_to": "paddle.nn.Conv3DTranspose",
        "args_list": [
            "num_channels",
            "num_filters",
            "filter_size",
            "padding",
            "stride",
            "dilation",
            "groups",
            "param_attr",
            "bias_attr",
            "use_cudnn",
            "act",
            "dtype"
        ],
        "args_change": [
            [
                "num_channels",
                "in_channels"
            ],
            [
                "num_filters",
                "out_channels"
            ],
            [
                "filter_size",
                "kernel_size"
            ],
            [
                "param_attr",
                "weight_attr"
            ],
            [
                "use_cudnn",
                ""
            ],
            [
                "act",
                ""
            ],
            [
                "dtype",
                ""
            ]
        ],
        "args_warning": {
            "use_cudnn": "this args is deleted in paddle.nn.Conv3DTranspose",
            "act": "this args is deleted in paddle.nn.Conv3DTranspose",
            "dtype": "this args is deleted in paddle.nn.Conv3DTranspose"
        }
    },
    "paddle.fluid.layers.nn.crop_tensor": {
        "alias": [
            "paddle.fluid.layers.crop_tensor"
        ],
        "update_to": "paddle.crop"
    },
    "paddle.fluid.layers.tensor.zeros": {
        "alias": [
            "paddle.fluid.layers.zeros"
        ],
        "update_to": "paddle.zeros",
        "args_list": [
            "shape",
            "dtype",
            "force_cpu"
        ],
        "args_change": [
            [
                "force_cpu",
                ""
            ]
        ],
        "args_warning": {
            "force_cpu": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.layers.reduce_prod": {
        "alias": [
            "paddle.fluid.layers.nn.reduce_prod"
        ],
        "update_to": "paddle.prod",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    "paddle.fluid.layers.elementwise_floordiv": {
        "alias": [
            "paddle.fluid.layers.nn.elementwise_floordiv"
        ],
        "update_to": "paddle.floor_divide",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "axis",
                ""
            ],
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "axis": "axis is deleted in paddle.floor_divide",
            "act": "act is deleted in paddle.floor_divide"
        }
    },
    "paddle.fluid.layers.elementwise_add": {
        "alias": [
            "paddle.fluid.layers.nn.elementwise_add"
        ],
        "update_to": "paddle.add",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "axis",
                ""
            ],
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "axis": "axis is deleted in paddle.add",
            "act": "act is deleted in paddle.add"
        }
    },
    "paddle.fluid.layers.nn.elementwise_sub": {
        "alias": [
            "paddle.fluid.layers.elementwise_sub"
        ],
        "warning": "please use paddle.add to do subtract."
    },
    # manual check
    "paddle.fluid.layers.elementwise_mul": {
        "alias": [
            "paddle.fluid.layers.nn.elementwise_mul"
        ],
        "update_to": "paddle.multiply",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "act": "act is deleted in paddle.multiply"
        }
    },
    "paddle.fluid.layers.elementwise_equal": {
        "alias": [
            "paddle.fluid.layers.nn.elementwise_equal"
        ],
        "update_to": "paddle.equal"
    },
    # TODO transformer
    "paddle.fluid.layers.hsigmoid": {
        "alias": [
            "paddle.fluid.layers.loss.hsigmoid"
        ],
        "warning": "this api is update to paddle.nn.functional.hsigmoid_loss",
    },
    "paddle.fluid.layers.scatter_nd": {
        "alias": [
            "paddle.fluid.layers.nn.scatter_nd"
        ],
        "update_to": "paddle.scatter_nd"
    },
    "paddle.fluid.dygraph.LayerList": {
        "alias": [
            "paddle.fluid.dygraph.container.LayerList"
        ],
        "update_to": "paddle.nn.LayerList"
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.PiecewiseDecay": {
        "alias": [
            "paddle.fluid.dygraph.PiecewiseDecay"
        ],
        "warning": "this api is update to paddle.optimizer.lr.PiecewiseDecay"
    },
    "paddle.fluid.CPUPlace": {
        "update_to": "paddle.CPUPlace"
    },
    "paddle.fluid.layers.pixel_shuffle": {
        "alias": [
            "paddle.fluid.layers.nn.pixel_shuffle"
        ],
        "update_to": "paddle.nn.functional.pixel_shuffle",
    },
    "paddle.fluid.optimizer.MomentumOptimizer": {
        "alias": [
            "paddle.fluid.optimizer.Momentum"
        ],
        "update_to": "paddle.optimizer.Momentum",
        "args_list": [
            "learning_rate",
            "momentum",
            "parameter_list",
            "use_nesterov",
            "regularization",
            "grad_clip",
            "name"
        ],
        "args_change": [
            [
                "parameter_list",
                "parameters"
            ],
            [
                "regularization",
                "weight_decay"
            ]
        ]
    },
    "paddle.fluid.layers.tensor.ones_like": {
        "alias": [
            "paddle.fluid.layers.ones_like"
        ],
        "update_to": "paddle.ones_like",
        "args_list": [
            "x",
            "out"
        ],
        "args_change": [
            [
                "out",
                ""
            ]
        ],
        "args_warning": {
            "out": "this args is deleted in this version"
        }
    },
    "paddle.fluid.layers.nn.strided_slice": {
        "alias": [
            "paddle.fluid.layers.strided_slice"
        ],
        "update_to": "paddle.strided_slice",
        "args_list": [
            "input",
            "axes",
            "starts",
            "ends",
            "strides"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.StepDecay": {
        "alias": [
            "paddle.fluid.dygraph.StepDecay"
        ],
        "warning": "this api is update to paddle.optimizer.lr.StepDecay"
    },
    "paddle.fluid.layers.log": {
        "alias": [
            "paddle.fluid.layers.nn.log"
        ],
        "update_to": "paddle.log"
    },
    "paddle.fluid.layers.loss.nce": {
        "alias": [
            "paddle.fluid.layers.nce"
        ],
        "update_to": "paddle.static.nn.nce"
    },
    "paddle.fluid.layers.nn.flip": {
        "alias": [
            "paddle.fluid.layers.flip"
        ],
        "update_to": "paddle.flip",
        "args_list": [
            "input",
            "dims",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dims",
                "axis"
            ]
        ]
    },
    "paddle.fluid.layers.nn.shard_index": {
        "alias": [
            "paddle.fluid.layers.shard_index"
        ],
        "update_to": "paddle.shard_index"
    },
    "paddle.fluid.layers.elementwise_mod": {
        "alias": [
            "paddle.fluid.layers.nn.elementwise_mod"
        ],
        "update_to": "paddle.mod",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "axis",
                ""
            ],
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "axis": "axis is deleted in paddle.mod",
            "act": "act is deleted in paddle.mod"
        }
    },
     # manual check
    "paddle.fluid.dygraph.nn.Linear": {
        "alias": [
            "paddle.fluid.dygraph.Linear"
        ],
        "update_to": "paddle.nn.Linear",
        "args_list": [
            "input_dim",
            "output_dim",
            "param_attr",
            "bias_attr",
            "act",
            "dtype"
        ],
        "args_change": [
            [
                "input_dim",
                "in_features"
            ],
            [
                "output_dim",
                "out_features"
            ],
            [
                "param_attr",
                "weight_attr"
            ],
            [
                "act",
                ""
            ],
            [
                "dtype",
                ""
            ]
        ],
        "args_warning": {
            "act": "this args is deleted in paddle.nn.Linear",
            "dtype": "this args is deleted in paddle.nn.Linear"
        }
    },
    "paddle.fluid.layers.clip_by_norm": {
        "alias": [
            "paddle.fluid.layers.nn.clip_by_norm"
        ],
        "update_to": "paddle.nn.clip_by_norm"
    },
    "paddle.fluid.regularizer.L2DecayRegularizer": {
        "alias": [
            "paddle.fluid.regularizer.L2Decay"
        ],
        "update_to": "paddle.regularizer.L2Decay",
        "args_list": [
            "regularization_coeff"
        ],
        "args_change": [
            [
                "regularization_coeff",
                "coeff"
            ]
        ]
    },
    "paddle.fluid.layers.chunk_eval": {
        "alias": [
            "paddle.fluid.layers.nn.chunk_eval"
        ],
        "update_to": "paddle.metric.chunk_eval"
    },
    "paddle.fluid.layers.tensor.argsort": {
        "alias": [
            "paddle.fluid.layers.argsort"
        ],
        "update_to": "paddle.argsort",
        "args_list": [
            "input",
            "axis",
            "descending",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.layers.nn.prelu": {
        "alias": [
            "paddle.fluid.layers.prelu"
        ],
        "update_to": "paddle.static.nn.prelu"
    },
    "paddle.fluid.layers.ops.reciprocal": {
        "alias": [
            "paddle.fluid.layers.reciprocal"
        ],
        "update_to": "paddle.reciprocal"
    },
    "paddle.fluid.layers.nn.logical_xor": {
        "alias": [
            "paddle.fluid.layers.logical_xor"
        ],
        "update_to": "paddle.logical_xor"
    },
    "paddle.fluid.dygraph.dygraph_to_static.create_static_variable_gast_node": {
        "alias": [
            "paddle.fluid.dygraph.dygraph_to_static.variable_trans_func.create_static_variable_gast_node"
        ],
        "update_to": "paddle.jit.dy2static.create_static_variable_gast_node"
    },
    "paddle.fluid.layers.crf_decoding": {
        "alias": [
            "paddle.fluid.layers.nn.crf_decoding"
        ],
        "update_to": "paddle.static.nn.crf_decoding"
    },
    # TODO transformer
    "paddle.fluid.layers.diag": {
        "alias": [
            "paddle.fluid.layers.tensor.diag"
        ],
        "warning": "this api is update to paddle.diag"
    },
    "paddle.fluid.layers.less_than": {
        "alias": [
            "paddle.fluid.layers.control_flow.less_than"
        ],
        "update_to": "paddle.less_than",
        "args_list": [
            "x",
            "y",
            "force_cpu",
            "cond"
        ],
        "args_change": [
            [
                "force_cpu",
                ""
            ],
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "force_cpu": "this args is deleted in this version",
            "cond": "this args is deleted in this version"
        }
    },
    "paddle.fluid.layers.affine_grid": {
        "alias": [
            "paddle.fluid.layers.nn.affine_grid"
        ],
        "update_to": "paddle.nn.functional.affine_grid",
        "args_list": [
            "theta",
            "out_shape",
            "name"
        ],
    },
    "paddle.fluid.layers.nn.spectral_norm": {
        "alias": [
            "paddle.fluid.layers.spectral_norm"
        ],
        "update_to": "paddle.static.nn.spectral_norm"
    },
    "paddle.fluid.layers.transpose": {
        "alias": [
            "paddle.fluid.layers.nn.transpose",
            "paddle.complex.transpose",
            "paddle.complex.tensor.transpose",
            "paddle.complex.tensor.manipulation.transpose"
        ],
        "update_to": "paddle.transpose"
    },
    "paddle.fluid.layers.nn.nonzero": {
        "alias": [
            "paddle.fluid.layers.nonzero"
        ],
        "update_to": "paddle.nonzero",
    },
    "paddle.fluid.layers.switch_case": {
        "alias": [
            "paddle.fluid.layers.control_flow.switch_case"
        ],
        "update_to": "paddle.static.nn.switch_case"
    },
    "paddle.fluid.framework.Variable": {
        "alias": [
            "paddle.fluid.Variable"
        ],
        "update_to": "paddle.static.Variable"
    },
    "paddle.fluid.layers.atan": {
        "alias": [
            "paddle.fluid.layers.ops.atan"
        ],
        "update_to": "paddle.atan"
    },
    "paddle.fluid.dataloader.Dataset": {
        "alias": [
            "paddle.fluid.dataloader.dataset.Dataset"
        ],
        "update_to": "paddle.io.Dataset"
    },
    "paddle.fluid.layers.nn.cross": {
        "alias": [
            "paddle.fluid.layers.cross"
        ],
        "update_to": "paddle.cross",
        "args_list": [
            "input",
            "other",
            "dim"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "other",
                "y"
            ],
            [
                "dim",
                "axis"
            ]
        ]
    },
    "paddle.fluid.layers.case": {
        "alias": [
            "paddle.fluid.layers.control_flow.case"
        ],
        "update_to": "paddle.static.nn.case"
    },
    "paddle.fluid.layers.fill_constant": {
        "alias": [
            "paddle.fluid.layers.tensor.fill_constant"
        ],
        "update_to": "paddle.full",
        "args_list": [
            "shape",
            "dtype",
            "value",
            "force_cpu",
            "out"
        ],
        "args_change": [
            [
                "shape",
                "shape"
            ],
            [
                "dtype",
                ""
            ],
            [
                "value",
                "fill_value"
            ],
            [
                "force_cpu",
                ""
            ],
            [
                "out",
                ""
            ]
        ],
    },
    "paddle.fluid.layers.nn.crop": {
        "alias": [
            "paddle.fluid.layers.crop"
        ],
        "update_to": "paddle.crop",
    },
    "paddle.fluid.layers.shape": {
        "alias": [
            "paddle.fluid.layers.nn.shape"
        ],
        "update_to": "paddle.shape"
    },
    "paddle.fluid.layers.ones": {
        "alias": [
            "paddle.fluid.layers.tensor.ones"
        ],
        "update_to": "paddle.ones",
        "args_list": [
            "shape",
            "dtype",
            "force_cpu"
        ],
        "args_change": [
            [
                "force_cpu",
                ""
            ]
        ]
    },
    # TODO transformer
    "paddle.fluid.dygraph.LayerNorm": {
        "alias": [
            "paddle.fluid.dygraph.nn.LayerNorm"
        ],
        "warning": "this api is update to paddle.nn.LayerNorm",
    },
    "paddle.fluid.optimizer.AdagradOptimizer": {
        "alias": [
            "paddle.fluid.optimizer.Adagrad"
        ],
        "update_to": "paddle.optimizer.Adagrad",
    },
    "paddle.fluid.layers.layer_norm": {
        "alias": [
            "paddle.fluid.layers.nn.layer_norm"
        ],
        "update_to": "paddle.static.nn.layer_norm"
    },
    "paddle.fluid.layers.one_hot": {
        "alias": [
            "paddle.fluid.layers.nn.one_hot",
            "paddle.fluid.one_hot",
            "paddle.fluid.input.one_hot"
        ],
        "warning": "input->x, depth->num_classes, x'elements must less than num_classes."
    },
    "paddle.fluid.layers.argmin": {
        "alias": [
            "paddle.fluid.layers.tensor.argmin"
        ],
        "update_to": "paddle.argmin",
    },
    "paddle.fluid.layers.square": {
        "alias": [
            "paddle.fluid.layers.ops.square"
        ],
        "update_to": "paddle.square"
    },
    "paddle.fluid.layers.io.data": {
        "alias": [
            "paddle.fluid.layers.data"
        ],
        "update_to": "paddle.static.data",
        "args_list": [
            "name",
            "shape",
            "append_batch_size",
            "dtype",
            "lod_level",
            "type",
            "stop_gradient"
        ],
        "args_change": [
            [
                "append_batch_size",
                ""
            ],
            [
                "type",
                ""
            ],
            [
                "stop_gradient",
                ""
            ]
        ],
        "args_warning": {
            "append_batch_size": "This args is deleted in this version.",
            "type": "This args is deleted in this version.",
            "stop_gradient": "This args is deleted in this version.",
        }
    },
    "paddle.fluid.layers.loss.kldiv_loss": {
        "alias": [
            "paddle.fluid.layers.kldiv_loss"
        ],
        "update_to": "paddle.nn.functional.kl_div",
        "args_list": [
            "x",
            "target",
            "reduction",
            "name"
        ],
        "args_change": [
            [
                "x",
                "input"
            ],
            [
                "target",
                "label"
            ]
        ]
    },
    # TODO transformer
    "paddle.fluid.dataloader.BatchSampler": {
        "alias": [
            "paddle.fluid.dataloader.batch_sampler.BatchSampler"
        ],
        "warning": "this api is update to paddle.io.BatchSampler",
    },
    "paddle.fluid.layers.nn.logical_or": {
        "alias": [
            "paddle.fluid.layers.logical_or"
        ],
        "update_to": "paddle.logical_or"
    },
    "paddle.fluid.layers.less_equal": {
        "alias": [
            "paddle.fluid.layers.control_flow.less_equal"
        ],
        "update_to": "paddle.less_equal",
        "args_list": [
            "x",
            "y",
            "cond"
        ],
        "args_change": [
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "cond": "cond is deleted in this version"
        }
    },
    "paddle.fluid.dygraph.parallel.prepare_context": {
        "alias": [
            "paddle.fluid.dygraph.prepare_context"
        ],
        "update_to": "paddle.distributed.prepare_context"
    },
    "paddle.reader.decorator.multiprocess_reader": {
        "alias": [
            "paddle.fluid.io.multiprocess_reader",
            "paddle.reader.multiprocess_reader"
        ],
        "update_to": "paddle.reader.decorator.multiprocess_reader"
    },
    "paddle.fluid.layers.concat": {
        "alias": [
            "paddle.fluid.layers.tensor.concat"
        ],
        "update_to": "paddle.concat",
        "args_list": [
            "input",
            "axis",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.in_dygraph_mode": {
        "alias": [
            "paddle.fluid.framework.in_dygraph_mode"
        ],
        "update_to": "paddle.in_dynamic_mode"
    },
    "paddle.fluid.dygraph.MSELoss": {
        "alias": [
            "paddle.fluid.dygraph.nn.MSELoss"
        ],
        "update_to": "paddle.nn.MSELoss"
    },
    "paddle.fluid.framework.name_scope": {
        "alias": [
            "paddle.fluid.name_scope"
        ],
        "update_to": "paddle.static.name_scope"
    },
    "paddle.fluid.dygraph.NLLLoss": {
        "alias": [
            "paddle.fluid.dygraph.nn.NLLLoss"
        ],
        "update_to": "paddle.nn.NLLLoss",
    },
    "paddle.fluid.layers.ops.softsign": {
        "alias": [
            "paddle.fluid.layers.softsign"
        ],
        "update_to": "paddle.nn.functional.softsign"
    },
    "paddle.fluid.layers.t": {
        "alias": [
            "paddle.fluid.layers.nn.t"
        ],
        "update_to": "paddle.t"
    },
    "paddle.fluid.layers.nn.selu": {
        "alias": [
            "paddle.fluid.layers.selu"
        ],
        "update_to": "paddle.nn.functional.selu"
    },
    "paddle.fluid.layers.meshgrid": {
        "alias": [
            "paddle.fluid.layers.nn.meshgrid"
        ],
        "update_to": "paddle.meshgrid",
    },
    "paddle.fluid.dygraph.nn.Conv3D": {
        "alias": [
            "paddle.fluid.dygraph.Conv3D"
        ],
        "update_to": "paddle.nn.Conv3D",
        "args_list": [
            "num_channels",
            "num_filters",
            "filter_size",
            "stride",
            "padding",
            "dilation",
            "groups",
            "param_attr",
            "bias_attr",
            "use_cudnn",
            "act",
            "dtype"
        ],
        "args_change": [
            [
                "num_channels",
                "in_channels"
            ],
            [
                "num_filters",
                "out_channels"
            ],
            [
                "filter_size",
                "kernel_size"
            ],
            [
                "param_attr",
                "weight_attr"
            ],
            [
                "use_cudnn",
                ""
            ],
            [
                "act",
                ""
            ],
            [
                "dtype",
                ""
            ]
        ],
        "args_warning": {
            "use_cudnn": "this args is deleted in paddle.nn.Conv3d",
            "act": "this args is deleted in paddle.nn.Conv3d",
            "dtype": "this args is deleted in paddle.nn.Conv3d"
        }
    },
    "paddle.fluid.layers.exp": {
        "alias": [
            "paddle.fluid.layers.ops.exp"
        ],
        "update_to": "paddle.exp"
    },
    "paddle.fluid.backward.gradients": {
        "alias": [
            "paddle.fluid.gradients"
        ],
        "update_to": "paddle.static.gradients"
    },
    "paddle.fluid.layers.control_flow.equal": {
        "alias": [
            "paddle.fluid.layers.equal"
        ],
        "update_to": "paddle.equal",
        "args_list": [
            "x",
            "y",
            "cond"
        ],
        "args_change": [
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "cond": "this args is deleted in this version"
        }
    },
    "paddle.fluid.ComplexVariable": {
        "alias": [
            "paddle.fluid.framework.ComplexVariable"
        ],
        "warning": "this api is update to paddle.ComplexTensor",
    },
    "paddle.fluid.layers.split": {
        "alias": [
            "paddle.fluid.layers.nn.split"
        ],
        "update_to": "paddle.split",
        "args_list": [
            "input",
            "num_or_sections",
            "dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ]
        ]
    },
    "paddle.fluid.layers.dynamic_decode": {
        "update_to": "paddle.nn.dynamic_decode"
    },
    "paddle.fluid.layers.kron": {
        "alias": [
            "paddle.fluid.layers.tensor.kron"
        ],
        "update_to": "paddle.kron",
        "args_list": [
            "x",
            "y",
            "out",
            "name"
        ],
        "args_change": [
            [
                "out",
                ""
            ]
        ]
    },
    "paddle.fluid.layers.unfold": {
        "alias": [
            "paddle.fluid.layers.nn.unfold"
        ],
        "update_to": "paddle.nn.functional.unfold"
    },
    "paddle.fluid.layers.data_norm": {
        "alias": [
            "paddle.fluid.layers.nn.data_norm"
        ],
        "update_to": "paddle.static.nn.data_norm",
    },
    "paddle.fluid.dygraph.jit.TracedLayer": {
        "alias": [
            "paddle.fluid.dygraph.TracedLayer"
        ],
        "update_to": "paddle.jit.TracedLayer"
    },
    "paddle.fluid.ParamAttr": {
        "alias": [
            "paddle.fluid.param_attr.ParamAttr"
        ],
        "update_to": "paddle.ParamAttr",
    },
    "paddle.fluid.layers.nn.bilinear_tensor_product": {
        "alias": [
            "paddle.fluid.layers.bilinear_tensor_product"
        ],
        "update_to": "paddle.static.nn.bilinear_tensor_product"
    },
    "paddle.fluid.layers.round": {
        "alias": [
            "paddle.fluid.layers.ops.round"
        ],
        "update_to": "paddle.round"
    },
    "paddle.fluid.layers.mean": {
        "alias": [
            "paddle.fluid.layers.nn.mean"
        ],
        "update_to": "paddle.mean",
        "args_list": [
            "x",
            "name"
        ]
    },
    "paddle.fluid.io.save_inference_model": {
        "update_to": "paddle.static.save_inference_model"
    },
    "paddle.fluid.framework.cpu_places": {
        "alias": [
            "paddle.fluid.cpu_places"
        ],
        "update_to": "paddle.static.cpu_places"
    },
    "paddle.fluid.executor.global_scope": {
        "alias": [
            "paddle.fluid.global_scope"
        ],
        "update_to": "paddle.static.global_scope"
    },
    # TODO transformer
    "paddle.fluid.layers.where": {
        "alias": [
            "paddle.fluid.layers.nn.where"
        ],
        "warning": "this api is update to paddle.where",
    },
    # TODO transformer
    "paddle.fluid.layers.batch_norm": {
        "alias": [
            "paddle.fluid.layers.nn.batch_norm"
        ],
        "warning": " this api is update to paddle.nn.functional.batch_norm"
    },
    "paddle.fluid.layers.slice": {
        "alias": [
            "paddle.fluid.layers.nn.slice"
        ],
        "update_to": "paddle.slice"
    },
    "paddle.fluid.layers.addmm": {
        "alias": [
            "paddle.fluid.layers.nn.addmm"
        ],
        "update_to": "paddle.addmm",
        "args_list": [
            "input",
            "x",
            "y",
            "alpha",
            "beta",
            "name"
        ],
    },
    "paddle.fluid.layers.scatter": {
        "alias": [
            "paddle.fluid.layers.nn.scatter"
        ],
        "update_to": "paddle.scatter",
        "args_list": [
            "input",
            "index",
            "updates",
            "name",
            "overwrite"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.layers.nn.index_select": {
        "alias": [
            "paddle.fluid.layers.index_select"
        ],
        "update_to": "paddle.index_select",
        "args_list": [
            "input",
            "index",
            "dim"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ]
        ]
    },
    "paddle.fluid.layers.index_sample": {
        "alias": [
            "paddle.fluid.layers.nn.index_sample"
        ],
        "update_to": "paddle.index_sample"
    },
    "paddle.fluid.WeightNormParamAttr": {
        "alias": [
            "paddle.fluid.param_attr.WeightNormParamAttr"
        ],
        "update_to": "paddle.static.WeightNormParamAttr"
    },
    "paddle.fluid.dygraph.no_grad": {
        "alias": [
            "paddle.fluid.dygraph.base.no_grad"
        ],
        "update_to": "paddle.framework.no_grad"
    },
    # TODO transformer paddle.nn.functional.dropout
    "paddle.fluid.layers.nn.dropout": {
        "alias": [
            "paddle.fluid.layers.dropout"
        ],
        "warning": "this api is update to paddle.nn.functional.dropout",
    },
    "paddle.fluid.layers.cast": {
        "alias": [
            "paddle.fluid.layers.tensor.cast"
        ],
        "update_to": "paddle.cast"
    },
    "paddle.fluid.layers.label_smooth": {
        "alias": [
            "paddle.fluid.layers.nn.label_smooth"
        ],
        "update_to": "paddle.nn.functional.label_smooth",
    },
    "paddle.fluid.framework.default_main_program": {
        "alias": [
            "paddle.fluid.default_main_program"
        ],
        "update_to": "paddle.static.default_main_program"
    },
    "paddle.fluid.program_guard": {
        "alias": [
            "paddle.fluid.framework.program_guard"
        ],
        "update_to": "paddle.static.program_guard"
    },
    "paddle.fluid.layers.Print": {
        "alias": [
            "paddle.fluid.layers.control_flow.Print"
        ],
        "update_to": "paddle.static.Print",
        "args_list": [
            "input",
            "first_n",
            "message",
            "summarize",
            "print_tensor_name",
            "print_tensor_type",
            "print_tensor_shape",
            "print_tensor_lod",
            "print_phase"
        ]
    },
    "paddle.fluid.dataset.InMemoryDataset": {
        "update_to": "paddle.distributed.InMemoryDataset"
    },
    # manual check
    "paddle.fluid.layers.squeeze": {
        "alias": [
            "paddle.fluid.layers.nn.squeeze"
        ],
        "update_to": "paddle.squeeze",
        "args_list": [
            "input",
            "axes",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "axes",
                "axis"
            ]
        ]
    },
    "paddle.fluid.dygraph.BatchNorm": {
        "alias": [
            "paddle.fluid.dygraph.nn.BatchNorm"
        ],
        "update_to": "paddle.nn.BatchNorm"
    },
    "paddle.fluid.layers.elu": {
        "alias": [
            "paddle.fluid.layers.nn.elu"
        ],
        "update_to": "paddle.nn.functional.elu"
    },
    "paddle.fluid.dygraph.nn.Embedding": {
        "alias": [
            "paddle.fluid.dygraph.Embedding"
        ],
        "warning": "this api is update to paddle.nn.Embedding"
    },
    "paddle.fluid.dygraph.Conv2DTranspose": {
        "alias": [
            "paddle.fluid.dygraph.nn.Conv2DTranspose"
        ],
        "warning": "this api is update to paddle.nn.Conv2DTranspose"
    },
    "paddle.fluid.layers.nn.bmm": {
        "alias": [
            "paddle.fluid.layers.bmm"
        ],
        "update_to": "paddle.bmm"
    },
    "paddle.fluid.layers.reduce_sum": {
        "alias": [
            "paddle.fluid.layers.nn.reduce_sum"
        ],
        "update_to": "paddle.sum",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    "paddle.fluid.initializer.TruncatedNormalInitializer": {
        "alias": [
            "paddle.fluid.initializer.TruncatedNormal"
        ],
        "warning": "this api is update to paddle.nn.initializer.TruncatedNormal",
    
    },
    "paddle.fluid.layers.maxout": {
        "alias": [
            "paddle.fluid.layers.nn.maxout"
        ],
        "update_to": "paddle.nn.functional.maxout",
    },
    "paddle.fluid.dygraph.parallel.ParallelEnv": {
        "alias": [
            "paddle.fluid.dygraph.ParallelEnv"
        ],
        "update_to": "paddle.distributed.ParallelEnv"
    },
    "paddle.fluid.layers.triu": {
        "alias": [
            "paddle.fluid.layers.tensor.triu"
        ],
        "update_to": "paddle.triu",
        "args_list": [
            "input",
            "diagonal",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.layers.nn.scatter_nd_add": {
        "alias": [
            "paddle.fluid.layers.scatter_nd_add"
        ],
        "update_to": "paddle.scatter_nd_add",
    },
    "paddle.fluid.ExecutionStrategy": {
        "alias": [
            "paddle.fluid.compiler.ExecutionStrategy"
        ],
        "update_to": "paddle.static.ExecutionStrategy"
    },
    "paddle.fluid.layers.nn.conv3d_transpose": {
        "alias": [
            "paddle.fluid.layers.conv3d_transpose"
        ],
        "warning": "this api is update to paddle.nn.functional.conv3d_transpose",
    },
    "paddle.fluid.io.load_program_state": {
        "update_to": "paddle.static.load_program_state"
    },
    "paddle.fluid.layers.accuracy": {
        "alias": [
            "paddle.fluid.layers.metric_op.accuracy"
        ],
        "update_to": "paddle.metric.accuracy",
    },
    "paddle.fluid.layers.abs": {
        "alias": [
            "paddle.fluid.layers.ops.abs"
        ],
        "update_to": "paddle.abs"
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.NoamDecay": {
        "alias": [
            "paddle.fluid.dygraph.NoamDecay"
        ],
        "warning": "this api is update to paddle.optimizer.lr.NoamDecay",

    },
    "paddle.fluid.layers.erf": {
        "alias": [
            "paddle.fluid.layers.ops.erf"
        ],
        "update_to": "paddle.erf",
    },
    "paddle.fluid.framework.cuda_places": {
        "alias": [
            "paddle.fluid.cuda_places"
        ],
        "update_to": "paddle.static.cuda_places"
    },
    # TODO transformer
    "paddle.fluid.layers.ops.cumsum": {
        "alias": [
            "paddle.fluid.layers.cumsum"
        ],
        "warning": "this api is update to paddle.cumsum",
    },
    "paddle.fluid.layers.nn.log_loss": {
        "alias": [
            "paddle.fluid.layers.log_loss"
        ],
        "update_to": "paddle.nn.functional.log_loss"
    },
    # TODO transformer
    "paddle.fluid.layers.nn.interpolate": {
        "alias": [
            "paddle.fluid.layers.interpolate"
        ],
        "warning": "this api is update to paddle.nn.functional.interpolate",
    },
    "paddle.fluid.dygraph.nn.L1Loss": {
        "alias": [
            "paddle.fluid.dygraph.L1Loss"
        ],
        "update_to": "paddle.nn.L1Loss",
    },
    "paddle.fluid.io.batch": {
        "alias": [
            "paddle.batch"
        ],
        "update_to": "paddle.batch"
    },
    "paddle.fluid.layers.nn.gather_nd": {
        "alias": [
            "paddle.fluid.layers.gather_nd"
        ],
        "update_to": "paddle.gather_nd",
        "args_list": [
            "input",
            "index",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.layers.nn.row_conv": {
        "alias": [
            "paddle.fluid.layers.row_conv"
        ],
        "update_to": "paddle.static.nn.row_conv"
    },
    "paddle.fluid.layers.unstack": {
        "alias": [
            "paddle.fluid.layers.nn.unstack"
        ],
        "update_to": "paddle.unstack"
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.LambdaDecay": {
        "alias": [
            "paddle.fluid.dygraph.LambdaDecay"
        ],
        "warning": " this api is update to paddle.optimizer.lr.LambdaDecay"
    },
    "paddle.fluid.layers.nn.unique": {
        "alias": [
            "paddle.fluid.layers.unique"
        ],
        "update_to": "paddle.unique",
        "args_list": [
            "x",
            "dtype"
        ],
    },
    "paddle.fluid.dygraph.layers.Layer": {
        "alias": [
            "paddle.fluid.dygraph.Layer"
        ],
        "update_to": "paddle.nn.Layer"
    },
    "paddle.fluid.layers.acos": {
        "alias": [
            "paddle.fluid.layers.ops.acos"
        ],
        "update_to": "paddle.acos"
    },
    "paddle.fluid.layers.nn.roll": {
        "alias": [
            "paddle.fluid.layers.roll"
        ],
        "update_to": "paddle.roll",
        "args_list": [
            "input",
            "shifts",
            "dims"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dims",
                "axis"
            ]
        ]
    },
    "paddle.fluid.layers.nn.unsqueeze": {
        "alias": [
            "paddle.fluid.layers.unsqueeze"
        ],
        "update_to": "paddle.unsqueeze",
        "args_list": [
            "input",
            "axes",
            "name"
        ],
        "args_change": [
            [
                "input",
                "a"
            ],
            [
                "axes",
                "axis"
            ]
        ]
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.MultiStepDecay": {
        "alias": [
            "paddle.fluid.dygraph.MultiStepDecay"
        ],
        "warning": "this api is update to paddle.optimizer.lr.MultiStepDecay",
    },
    "paddle.fluid.layers.loss.mse_loss": {
        "alias": [
            "paddle.fluid.layers.mse_loss"
        ],
        "update_to": "paddle.nn.functional.mse_loss"
    },
    # manual check
    "paddle.fluid.layers.pad": {
        "alias": [
            "paddle.fluid.layers.nn.pad"
        ],
        "warning": " this api is update to paddle.nn.functional.pad",
    },
    # warning 
    "paddle.fluid.layers.cross_entropy": {
        "alias": [
            "paddle.fluid.layers.loss.cross_entropy"
        ],
        "warning": " this api is update to paddle.nn.functional.cross_entropy"
    },
    "paddle.fluid.layers.tensor.range": {
        "alias": [
            "paddle.fluid.layers.range"
        ],
        "update_to": "paddle.arange",
    },
    "paddle.fluid.layers.arange": {
        "alias": [
            "paddle.fluid.layers.tensor.arange"
        ],
        "update_to": "paddle.arange"
    },
    "paddle.fluid.layers.py_func": {
        "alias": [
            "paddle.fluid.layers.nn.py_func"
        ],
        "update_to": "paddle.static.py_func"
    },
    "paddle.fluid.layers.ops.floor": {
        "alias": [
            "paddle.fluid.layers.floor"
        ],
        "update_to": "paddle.floor"
    },
    "paddle.fluid.layers.nn.elementwise_max": {
        "alias": [
            "paddle.fluid.layers.elementwise_max"
        ],
        "update_to": "paddle.maximum",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "act": "this api is deleted in this version"
        }
    },
    "paddle.fluid.layers.nn.elementwise_div": {
        "alias": [
            "paddle.fluid.layers.elementwise_div"
        ],
        "update_to": "paddle.divide",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "axis",
                ""
            ],
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "axis": "axis is deleted in paddle.divide",
            "act": "act is deleted in paddle.divide"
        }
    },    
    # TODO transformer
    "paddle.fluid.layers.loss.margin_rank_loss": {
        "alias": [
            "paddle.fluid.layers.margin_rank_loss"
        ],
        "warning": "this api is update to paddle.nn.functional.margin_ranking_loss"
    },
    # manual check
    "paddle.fluid.layers.greater_equal": {
        "alias": [
            "paddle.fluid.layers.control_flow.greater_equal"
        ],
        "update_to": "paddle.greater_equal",
        "args_list": [
            "x",
            "y",
            "cond"
        ],
        "args_change": [
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "cond": "this args is deleted in this version"
        }
    },
    "paddle.fluid.layers.fc": {
        "alias": [
            "paddle.fluid.layers.nn.fc"
        ],
        "update_to": "paddle.static.nn.fc",
        "args_list": [
            "input",
            "size",
            "num_flatten_dims",
            "param_attr",
            "bias_attr",
            "act",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "param_attr",
                "weight_attr"
            ],
            [
                "act",
                "activation"
            ]
        ],
        "warning": "in static graph, this api is update to paddle.static.nn.fc, in dynamic graph, this api is update to paddle.nn.functional.linear"
    },
    "paddle.fluid.layers.nn.reduce_all": {
        "alias": [
            "paddle.fluid.layers.reduce_all"
        ],
        "update_to": "paddle.all",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    # manual check
    "paddle.fluid.layers.nn.reduce_max": {
        "alias": [
            "paddle.fluid.layers.reduce_max"
        ],
        "update_to": "paddle.max",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    "paddle.fluid.layers.relu": {
        "alias": [
            "paddle.fluid.layers.nn.relu"
        ],
        "update_to": "paddle.nn.functional.relu"
    },
    "paddle.fluid.layers.nn.dot": {
        "alias": [
            "paddle.fluid.layers.dot"
        ],
        "update_to": "paddle.dot"
    },
    "paddle.fluid.executor.Executor": {
        "alias": [
            "paddle.fluid.Executor"
        ],
        "update_to": "paddle.static.Executor"
    },
    "paddle.fluid.layers.BeamSearchDecoder": {
        "update_to": "paddle.nn.BeamSearchDecoder"
    },
    "paddle.fluid.framework.is_compiled_with_cuda": {
        "alias": [
            "paddle.fluid.is_compiled_with_cuda"
        ],
        "update_to": "paddle.is_compiled_with_cuda"
    },
    "paddle.fluid.layers.nn.leaky_relu": {
        "alias": [
            "paddle.fluid.layers.leaky_relu"
        ],
        "update_to": "paddle.nn.functional.leaky_relu",
        "args_list": [
            "x",
            "alpha",
            "name"
        ],
        "args_change": [
            [
                "alpha",
                "negative_slope"
            ]
        ]
    },
    "paddle.fluid.layers.nn.sign": {
        "alias": [
            "paddle.fluid.layers.sign"
        ],
        "update_to": "paddle.sign",
    },
    "paddle.fluid.dygraph.base.grad": {
        "alias": [
            "paddle.fluid.dygraph.grad"
        ],
        "update_to": "paddle.grad",
    },
    "paddle.fluid.layers.logsumexp": {
        "alias": [
            "paddle.fluid.layers.nn.logsumexp"
        ],
        "update_to": "paddle.logsumexp",
        "args_list": [
            "x",
            "dim",
            "keepdim",
            "out",
            "name"
        ],
        "args_change": [
            [
                "dim",
                "axis"
            ],
            [
                "out",
                ""
            ]
        ],
        "args_warning": {
            "out": "this arg is deleted in this version."
        }
    },
    "paddle.fluid.scope_guard": {
        "alias": [
            "paddle.fluid.executor.scope_guard"
        ],
        "update_to": "paddle.static.scope_guard"
    },
    "paddle.fluid.layers.nn.logical_and": {
        "alias": [
            "paddle.fluid.layers.logical_and"
        ],
        "update_to": "paddle.logical_and"
    },
    "paddle.fluid.layers.ops.cos": {
        "alias": [
            "paddle.fluid.layers.cos"
        ],
        "update_to": "paddle.cos"
    },
    # TODO transformer
    "paddle.fluid.layers.nn.conv2d": {
        "alias": [
            "paddle.fluid.layers.conv2d"
        ],
        "warning": "this api is update to paddle.nn.functional.conv2d",
    },
    "paddle.fluid.layers.nn.group_norm": {
        "alias": [
            "paddle.fluid.layers.group_norm"
        ],
        "update_to": "paddle.static.nn.group_norm"
    },
    "paddle.fluid.initializer.ConstantInitializer": {
        "alias": [
            "paddle.fluid.initializer.Constant"
        ],
        "update_to": "paddle.nn.initializer.Constant",
        "args_list": [
            "value",
            "force_cpu"
        ],
        "args_change": [
            [
                "force_cpu",
                ""
            ]
        ],
        "args_warning": {
            "force_cpu": "this args is deleted in paddle.nn.initializer.Constant"
        }
    },
    "paddle.fluid.layers.nn.reduce_min": {
        "alias": [
            "paddle.fluid.layers.reduce_min"
        ],
        "update_to": "paddle.min",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    "paddle.fluid.dygraph.nn.Conv2D": {
        "alias": [
            "paddle.fluid.dygraph.Conv2D"
        ],
        "update_to": "paddle.nn.Conv2D",
        "args_change": [
            [
                "num_channels",
                "in_channels"
            ],
            [
                "num_filters",
                "out_channels"
            ],
            [
                "filter_size",
                "kernel_size"
            ],
            [
                "param_attr",
                "weight_attr"
            ],
            [
                "use_cudnn",
                ""
            ],
            [
                "act",
                ""
            ],
            [
                "dtype",
                ""
            ]
        ],
        "args_warning": {
            "use_cudnn": "this args is deleted in paddle.nn.Conv2d",
            "act": "this args is deleted in paddle.nn.Conv2d",
            "dtype": "this args is deleted in paddle.nn.Conv2d"
        }
    },
    "paddle.fluid.layers.nn.dist": {
        "alias": [
            "paddle.fluid.layers.dist"
        ],
        "update_to": "paddle.dist"
    },
    "paddle.fluid.layers.addcmul": {
        "alias": [
            "paddle.fluid.layers.nn.addcmul"
        ],
        "update_to": "paddle.tensor.math.addcmul",
        "args_list": [
            "input",
            "tensor1",
            "tensor2",
            "value",
            "out",
            "name"
        ],
        "args_change": [
            [
                "out",
                ""
            ]
        ],    
        "args_warning": {
            "out": "this arg is deleted in this version."
        }
    },
    "paddle.fluid.default_startup_program": {
        "alias": [
            "paddle.fluid.framework.default_startup_program"
        ],
        "update_to": "paddle.static.default_startup_program"
    },
    "paddle.fluid.layers.hard_sigmoid": {
        "alias": [
            "paddle.fluid.layers.nn.hard_sigmoid"
        ],
        "update_to": "paddle.nn.functional.hardsigmoid",
        "args_list": [
            "x",
            "slope",
            "offset",
            "name"
        ],
        "args_change": [
            [
                "slope",
                ""
            ],
            [
                "offset",
                ""
            ]
        ],
        "args_warning": {
            "slope": "this arg is deleted in this version.",
            "offset": "this arg is deleted in this version.",
        }
    },
    "paddle.fluid.dygraph.ParameterList": {
        "alias": [
            "paddle.fluid.dygraph.container.ParameterList"
        ],
        "update_to": "paddle.nn.ParameterList"
    },
    "paddle.fluid.layers.expand_as": {
        "alias": [
            "paddle.fluid.layers.nn.expand_as"
        ],
        "update_to": "paddle.expand_as",
        "args_list": [
            "x",
            "target_tensor",
            "name"
        ],
        "args_change": [
            [
                "target_tensor",
                "y"
            ],
        ]
    },
    "paddle.fluid.layers.reduce_mean": {
        "alias": [
            "paddle.fluid.layers.nn.reduce_mean"
        ],
        "update_to": "paddle.mean",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    "paddle.fluid.dygraph.learning_rate_scheduler.PolynomialDecay": {
        "alias": [
            "paddle.fluid.dygraph.PolynomialDecay"
        ],
        "warning": "this api is update to paddle.optimizer.lr.PolynomialDecay"
    },
    "paddle.fluid.layers.nn.swish": {
        "alias": [
            "paddle.fluid.layers.swish"
        ],
        "update_to": "paddle.nn.functional.swish",
        "args_list": [
            "x",
            "beta",
            "name"
        ],
        "args_change": [
            [
                "beta",
                ""
            ]
        ],
        "args_warning": {
            "beta": "this arg is deleted in this version."
        }
    },
    "paddle.fluid.layers.tensor.tril": {
        "alias": [
            "paddle.fluid.layers.tril"
        ],
        "update_to": "paddle.tril",
        "args_list": [
            "input",
            "diagonal",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.metrics.Recall": {
        "update_to": "paddle.metric.Recall",
    },
    "paddle.fluid.io.set_program_state": {
        "update_to": "paddle.static.set_program_state"
    },
    "paddle.fluid.ParallelExecutor": {
        "alias": [
            "paddle.fluid.parallel_executor.ParallelExecutor"
        ],
        "update_to": "paddle.static.ParallelExecutor"
    },
    "paddle.fluid.CUDAPinnedPlace": {
        "update_to": "paddle.CUDAPinnedPlace"
    },
    "paddle.fluid.layers.nn.randperm": {
        "alias": [
            "paddle.fluid.layers.randperm"
        ],
        "update_to": "paddle.randperm",
        "args_list": [
            "n",
            "out",
            "dtype",
            "device",
            "stop_gradient",
            "seed"
        ],
        "args_change": [
            [
                "out",
                ""
            ],
            [
                "device",
                ""
            ],
            [
                "stop_gradient",
                ""
            ],
            [
                "seed",
                ""
            ]
        ],
        "args_warning": {
            "out": "this arg is deleted in this version",
            "device": "this arg is deleted in this version",
            "stop_gradient": "this arg is deleted in this version",
            "seed": "this arg is deleted in this version",
        }
    },
    "paddle.fluid.layers.dice_loss": {
        "alias": [
            "paddle.fluid.layers.nn.dice_loss"
        ],
        "update_to": "paddle.nn.functional.dice_loss"
    },
    "paddle.fluid.layers.distributions.Normal": {
        "alias": [
            "paddle.fluid.layers.Normal"
        ],
        "update_to": "paddle.distribution.Normal"
    },
    "paddle.fluid.layers.log_softmax": {
        "alias": [
            "paddle.fluid.layers.nn.log_softmax"
        ],
        "update_to": "paddle.nn.functional.log_softmax",
        "args_list": [
            "input",
            "axis",
            "dtype",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.dygraph.nn.GroupNorm": {
        "alias": [
            "paddle.fluid.dygraph.GroupNorm"
        ],
        "update_to": "paddle.nn.GroupNorm",
        "args_list": [
            "channels",
            "groups",
            "epsilon",
            "param_attr",
            "bias_attr",
            "act",
            "data_layout",
            "dtype"
        ],
        "args_change": [
            [
                "channels",
                "num_channels"
            ],
            [
                "groups",
                "num_groups"
            ],
            [
                "param_attr",
                "weight_attr"
            ],
            [
                "act",
                ""
            ],
            [
                "data_layout",
                "data_format"
            ],
            [
                "dtype",
                ""
            ]
        ],
        "args_warning": {
            "act": "this arg is deleted in this version",
            "dtype": "this arg is deleted in this version"
        }

    },
    "paddle.fluid.layers.zeros_like": {
        "alias": [
            "paddle.fluid.layers.tensor.zeros_like"
        ],
        "update_to": "paddle.zeros_like",
        "args_list": [
            "x",
            "out"
        ],
        "args_change": [
            [
                "out",
                ""
            ]
        ],
        "args_warning": {
            "out": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.layers.nn.relu6": {
        "alias": [
            "paddle.fluid.layers.relu6"
        ],
        "warning": "this api is update to paddle.nn.functional.relu6",
    },
    "paddle.fluid.layers.sigmoid_focal_loss": {
        "alias": [
            "paddle.fluid.layers.detection.sigmoid_focal_loss"
        ],
        "update_to": "paddle.nn.functional.sigmoid_focal_loss",
        "args_list": [
            "x",
            "label",
            "fg_num",
            "gamma",
            "alpha"
        ],
        "args_change": [
            [
                "x",
                "logit"
            ],
            [
                "fg_num",
                "normalizer"
            ]
        ],
        "warning": "this api default returned summed loss in this version, for more infomation, please see paddle.nn.functional.sigmoid_focal_loss"
    },
    "paddle.fluid.CUDAPlace": {
        "update_to": "paddle.CUDAPlace"
    },
    "paddle.fluid.layers.tanh": {
        "alias": [
            "paddle.fluid.layers.ops.tanh"
        ],
        "update_to": "paddle.tanh"
    },
    "paddle.fluid.layers.nn.instance_norm": {
        "alias": [
            "paddle.fluid.layers.instance_norm"
        ],
        "warning": "this api is update to paddle.nn.functional.instance_norm",
    },
    "paddle.fluid.layers.ops.sigmoid": {
        "alias": [
            "paddle.fluid.layers.sigmoid"
        ],
        "update_to": "paddle.nn.functional.sigmoid"
    },
    "paddle.fluid.layers.increment": {
        "alias": [
            "paddle.fluid.layers.control_flow.increment"
        ],
        "update_to": "paddle.increment",
        "args_list": [
            "x",
            "value",
            "in_place"
        ],
        "args_change": [
            [
                "in_place",
                ""
            ]
        ],
        "args_warning": {
            "in_place": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.layers.tensor.eye": {
        "alias": [
            "paddle.fluid.layers.eye"
        ],
        "update_to": "paddle.eye",
        "args_list": [
            "num_rows",
            "num_columns",
            "batch_shape",
            "dtype"
        ],
        "args_change": [
            [
                "batch_shape",
                ""
            ]
        ],
        "args_warning": {
            "batch_shape": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.layers.loss.softmax_with_cross_entropy": {
        "alias": [
            "paddle.fluid.layers.softmax_with_cross_entropy"
        ],
        "update_to": "paddle.nn.functional.softmax_with_cross_entropy"
    },
    "paddle.fluid.dygraph.container.Sequential": {
        "alias": [
            "paddle.fluid.dygraph.Sequential"
        ],
        "update_to": "paddle.nn.Sequential"
    },
    "paddle.fluid.dygraph.NaturalExpDecay": {
        "alias": [
            "paddle.fluid.dygraph.learning_rate_scheduler.NaturalExpDecay"
        ],
        "warning": "this api is update to paddle.optimizer.lr.NaturalExpDecay",
    },
    "paddle.fluid.layers.cond": {
        "alias": [
            "paddle.fluid.layers.control_flow.cond"
        ],
        "update_to": "paddle.static.nn.cond"
    },
    "paddle.fluid.layers.elementwise_pow": {
        "alias": [
            "paddle.fluid.layers.nn.elementwise_pow"
        ],
        "update_to": "paddle.pow",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "axis",
                ""
            ],
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "axis": "this arg is deleted in this version",
            "act": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.layers.rank": {
        "alias": [
            "paddle.fluid.layers.nn.rank"
        ],
        "update_to": "paddle.rank"
    },
    "paddle.fluid.layers.isfinite": {
        "alias": [
            "paddle.fluid.layers.tensor.isfinite"
        ],
        "update_to": "paddle.isfinite",
    },
    "paddle.fluid.layers.nn.allclose": {
        "alias": [
            "paddle.fluid.layers.allclose"
        ],
        "update_to": "paddle.allclose",
        "args_list": [
            "input",
            "other",
            "rtol",
            "atol",
            "equal_nan",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "other",
                "y"
            ]
        ]
    },
    "paddle.fluid.metrics.Accuracy": {
        "update_to": "paddle.metric.Accuracy",
    },
    "paddle.fluid.layers.assign": {
        "alias": [
            "paddle.fluid.layers.tensor.assign"
        ],
        "update_to": "paddle.nn.functional.assign",
    },
    "paddle.fluid.layers.sin": {
        "alias": [
            "paddle.fluid.layers.ops.sin"
        ],
        "update_to": "paddle.sin"
    },
    "paddle.fluid.dygraph.BilinearTensorProduct": {
        "alias": [
            "paddle.fluid.dygraph.nn.BilinearTensorProduct"
        ],
        "update_to": "paddle.nn.layer.BilinearTensorProduct"
    },
    "paddle.fluid.layers.nn.topk": {
        "alias": [
            "paddle.fluid.layers.topk"
        ],
        "update_to": "paddle.topk",
        "args_list": [
            "input",
            "k",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ]
        ]
    },
    "paddle.fluid.dygraph.nn.Dropout": {
        "alias": [
            "paddle.fluid.dygraph.Dropout"
        ],
        "warning": "this api is update to paddle.nn.Dropout",
    },
    "paddle.fluid.layers.control_flow.not_equal": {
        "alias": [
            "paddle.fluid.layers.not_equal"
        ],
        "update_to": "paddle.not_equal",
        "args_list": [
            "x",
            "y",
            "cond"
        ],
        "args_change": [
            [
                "cond",
                ""
            ]
        ],
        "args_warning": {
            "cond": "this args is deleted in this version"
        }
    },
    "paddle.fluid.layers.nn.conv3d": {
        "alias": [
            "paddle.fluid.layers.conv3d"
        ],
        "warning": "this api is update to paddle.nn.functional.conv3d",
    },
    "paddle.fluid.dataset.QueueDataset": {
        "update_to": "paddle.distributed.QueueDataset"
    },
    "paddle.fluid.layers.iou_similarity": {
        "alias": [
            "paddle.fluid.layers.detection.iou_similarity"
        ],
        "update_to": "paddle.nn.functional.iou_similarity"
    },
    "paddle.fluid.metrics.Precision": {
        "update_to": "paddle.metric.Precision",
    },
    "paddle.fluid.layers.loss.npair_loss": {
        "alias": [
            "paddle.fluid.layers.npair_loss"
        ],
        "update_to": "paddle.nn.functional.npair_loss"
    },
    "paddle.fluid.layers.nn.stanh": {
        "alias": [
            "paddle.fluid.layers.stanh"
        ],
        "update_to": "paddle.stanh"
    },
    "paddle.fluid.require_version": {
        "alias": [
            "paddle.fluid.framework.require_version"
        ],
        "update_to": "paddle.utils.require_version"
    },
    "paddle.fluid.layers.LSTMCell": {
        "warning": "this api is update to paddle.nn.LSTMCell",
    },
    "paddle.fluid.layers.nn.conv2d_transpose": {
        "alias": [
            "paddle.fluid.layers.conv2d_transpose"
        ],
        "warning": "this api is update to paddle.nn.functional.conv2d_transpose"
    },
    "paddle.fluid.layers.nn.elementwise_min": {
        "alias": [
            "paddle.fluid.layers.elementwise_min"
        ],
        "update_to": "paddle.minimum",
        "args_list": [
            "x",
            "y",
            "axis",
            "act",
            "name"
        ],
        "args_change": [
            [
                "act",
                ""
            ]
        ],
        "args_warning": {
            "act": "this arg is deleted in this version"
        }
    },
    "paddle.fluid.layers.sqrt": {
        "alias": [
            "paddle.fluid.layers.ops.sqrt"
        ],
        "update_to": "paddle.sqrt"
    },
    "paddle.fluid.regularizer.L1DecayRegularizer": {
        "alias": [
            "paddle.fluid.regularizer.L1Decay"
        ],
        "update_to": "paddle.regularizer.L1Decay",
        "args_list": [
            "regularization_coeff"
        ],
        "args_change": [
            [
                "regularization_coeff",
                "coeff"
            ]
        ]
    },
    "paddle.fluid.layers.ops.thresholded_relu": {
        "alias": [
            "paddle.fluid.layers.thresholded_relu"
        ],
        "update_to": "paddle.nn.functional.thresholded_relu"
    },
    "paddle.fluid.layers.nn.log1p": {
        "alias": [
            "paddle.fluid.layers.log1p"
        ],
        "update_to": "paddle.log1p",
        "args_list": [
            "x",
            "out",
            "name"
        ],
        "args_change": [
            [
                "out",
                ""
            ]
        ],
        "args_warning": {
            "out": "this arg is deleted in this version."
        }
    },
    "paddle.fluid.layers.ops.gelu": {
        "alias": [
            "paddle.fluid.layers.gelu"
        ],
        "update_to": "paddle.nn.functional.gelu",
    },
    "paddle.fluid.layers.gather": {
        "alias": [
            "paddle.fluid.layers.nn.gather"
        ],
        "update_to": "paddle.gather",
        "args_list": [
            "input",
            "index",
            "overwrite"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "overwrite",
                ""
            ]
        ],
        "args_warning": {
            "overwrite": "this args is deleted in this version."
        }
    },
    "paddle.fluid.optimizer.AdamaxOptimizer": {
        "alias": [
            "paddle.fluid.optimizer.Adamax"
        ],
        "update_to": "paddle.optimizer.Adamax",
        "args_list": [
            "learning_rate",
            "beta1",
            "beta2",
            "epsilon",
            "parameter_list",
            "regularization",
            "grad_clip",
            "name"
        ],
        "args_change": [
            [
                "parameter_list",
                "parameters"
            ],
            [
                "regularization",
                "weight_decay"
            ]
        ]
    },
    "paddle.fluid.optimizer.AdamOptimizer": {
        "alias": [
            "paddle.fluid.optimizer.Adam"
        ],
        "update_to": "paddle.optimizer.Adam",
        "args_list": [
            "learning_rate",
            "beta1",
            "beta2",
            "epsilon",
            "parameter_list",
            "regularization",
            "grad_clip",
            "name",
            "lazy_mode"
        ],
        "args_change": [
            [
                "parameter_list",
                "parameters"
            ],
            [
                "regularization",
                "weight_decay"
            ]
        ]
    },
    "paddle.fluid.layers.ops.rsqrt": {
        "alias": [
            "paddle.fluid.layers.rsqrt"
        ],
        "update_to": "paddle.rsqrt"
    },
    "paddle.fluid.dygraph.DataParallel": {
        "alias": [
            "paddle.fluid.dygraph.parallel.DataParallel"
        ],
        "update_to": "paddle.DataParallel",
    },
    "paddle.fluid.layers.nn.stack": {
        "alias": [
            "paddle.fluid.layers.stack"
        ],
        "update_to": "paddle.stack",
    },
    "paddle.fluid.dygraph.nn.SpectralNorm": {
        "alias": [
            "paddle.fluid.dygraph.SpectralNorm"
        ],
        "update_to": "paddle.nn.SpectralNorm"
    },
    "paddle.fluid.layers.reduce_any": {
        "alias": [
            "paddle.fluid.layers.nn.reduce_any"
        ],
        "update_to": "paddle.any",
        "args_list": [
            "input",
            "dim",
            "keep_dim",
            "name"
        ],
        "args_change": [
            [
                "input",
                "x"
            ],
            [
                "dim",
                "axis"
            ],
            [
                "keep_dim",
                "keepdim"
            ]
        ]
    },
    "paddle.fluid.layers.GRUCell": {
        "warning": "this api is update to paddle.nn.GRUCell"
    }
}



import sys
import logging


__all__ = [
        'logger',
        'log_to_file',
        ]

class ColorFormatter(logging.Formatter):
    """Logging Formatter to add colors and count warning / errors"""
    def __init__(self, fmt=None, datefmt=None, style='%'):
        super().__init__(fmt, datefmt, style)
        self.fmt = fmt

        light_gray='\033[0;37m'
        dark_gray='\033[1;30m'
        yellow = "\033[0;33m"
        red = "\033[0;31m"
        reset = "\033[0m"

        self.FORMATS = {
            logging.DEBUG: dark_gray + fmt + reset,
            logging.INFO: light_gray + fmt + reset,
            logging.WARNING: yellow + fmt + reset,
            logging.ERROR: red + fmt + reset,
        }

    def format(self, record):
        log_fmt = None
        # if not windows, add color info
        if sys.platform.lower() != 'win32':
            log_fmt = self.FORMATS.get(record.levelno)
        if log_fmt is None:
            log_fmt = self.fmt
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)

def log_to_file(log_filepath="report.log"):
    log_filepath = log_filepath or "report.log"
    file_handler = logging.FileHandler(log_filepath)
    log_format = logging.Formatter('%(asctime)s - %(filename)s:%(lineno)d - %(levelname)s - %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    file_handler.setFormatter(log_format)
    logger.addHandler(file_handler)

def _build_default_logger():
    logger = logging.getLogger('paddle1to2')
    logger.setLevel("INFO")
    
    console_handler = logging.StreamHandler(stream=sys.stdout) # default stream is sys.stderr
    log_format = ColorFormatter('%(asctime)s - %(filename)s:%(lineno)d - %(levelname)s - %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    console_handler.setFormatter(log_format)
    logger.addHandler(console_handler)

    return logger

logger = _build_default_logger()



from bowler.types import LN, Capture, Filename, SYMBOL, TOKEN
from fissix.fixer_util import Name, Call, Number, KeywordArg, Comma, Newline
from fissix.pytree import Leaf, Node, type_repr
from fissix.pygram import python_grammar, python_symbols
from fissix.pgen2 import token
from fissix import patcomp

from paddle1to2 import utils
from paddle1to2.utils import log_debug, log_info, log_warning, log_error


def default_transformer(node: LN, capture: Capture, filename: Filename):
    fp = capture.get("function_parameters")
    if fp and fp.children[1].type == SYMBOL.arglist:
        arg_node = KeywordArg(Name("trans_arg"), Number("1"))
        fp.children[1].append_child(Comma())
        fp.children[1].append_child(arg_node)


def act_transformer(filename, trailer_node, removed_value):
    """
    add act to forward function, after delete act arg from api
    """
    if removed_value == "None":
        return
    # parent must be a power node
    power_node = trailer_node.parent
    if not isinstance(power_node, Node) and power_node.type != python_symbols.power:
        return
    # parent of parent must be an expression
    expr_node = power_node.parent
    if not isinstance(expr_node, Node) and expr_node.type != python_symbols.expr_stmt:
        return
    assign_idx = -1 # "=" index
    for idx in range(len(expr_node.children)):
        if expr_node.children[idx].type == token.EQUAL:
            assign_idx = idx
            break
    if assign_idx == -1:
        return
    layer_name = utils.node2code(expr_node.children[0:assign_idx])
    # Layer Class
    if 'self.' in layer_name:
        _forward_act_transformer(filename, expr_node, layer_name, removed_value)
    # invoke activation function directly
    else:
        _function_act_transformer(filename, expr_node, removed_value)

_pattern_funcdef_forward = "funcdef< 'def' 'forward' parameters< '(' ( 'self' | typedargslist< 'self' any* > ) any* ')' > any* >"
_pattern_funcdef_forward = patcomp.compile_pattern(_pattern_funcdef_forward)
_pattern_expr_stmt = "simple_stmt< expr_stmt< left=(any*) '=' right=(any*) > any* >"
_pattern_expr_stmt = patcomp.compile_pattern(_pattern_expr_stmt)

def _forward_act_transformer(filename, expr_node, layer_name, removed_value):
    # find funcdef node
    funcdef_node = None
    node = expr_node
    while node is not None:
        if node.type == python_symbols.funcdef:
            funcdef_node = node
            break
        node = node.parent
    if funcdef_node is None:
        return
    # find def forward function node
    forward_node = None
    classdef_node = None
    node = funcdef_node
    while node is not None:
        if node.type == python_symbols.classdef:
            classdef_node = node
            break
        node = node.parent
    if classdef_node is None:
        return
    for node in classdef_node.pre_order():
        results = {'node': node}
        if _pattern_funcdef_forward.match(node, results) and results["node"] is node:
            forward_node = node
            break
    if forward_node is None:
        return
    for node in forward_node.post_order():
        results = {'node': node}
        if _pattern_expr_stmt.match(node, results) and results["node"] is node:
            right=utils.node2code(results['right']).strip()
            if not utils.startswith(right, layer_name):
                continue
            left = utils.node2code(results['left']).strip()
            # if removed_value type is str
            if '"' in removed_value or "'" in removed_value:
                act = removed_value
                act = act.strip('"')
                act = act.strip("'")
                act = act.strip()
                # create statement like "x = paddle.nn.functional.act(x)"
                code = left + ' = ' + 'paddle.nn.functional.' + act + '(' + left + ')'
                _create_simple_stmt_node_and_insert_behind(code, node)
            # removed_value is a variable
            else:
                # add "self._act = act" after expr_node to make it visible to other methods
                act_var_name = "self._" + removed_value
                code = act_var_name + " = " + removed_value
                _create_simple_stmt_node_and_insert_behind(code, expr_node.parent)
                # create statement like "x = getattr(paddle.nn.functional, act)(x) if act else x"
                code = left + " = getattr(paddle.nn.functional, " + act_var_name + ")(" + left + ") if " + act_var_name + " else " + left
                _create_simple_stmt_node_and_insert_behind(code, node)
                log_warning(filename, expr_node.get_lineno(), 'variable "{}" may not be visible here.'.format(removed_value))


def _function_act_transformer(filename, expr_node, removed_value):
    simple_stmt_node = expr_node.parent
    results = {'node': simple_stmt_node}
    if _pattern_expr_stmt.match(simple_stmt_node, results) and results["node"] is simple_stmt_node:
        left = utils.node2code(results['left']).strip()
        # if removed_value type is str
        if '"' in removed_value or "'" in removed_value:
            act = removed_value
            act = act.strip('"')
            act = act.strip("'")
            act = act.strip()
            # create statement like "x = paddle.nn.functional.act(x)"
            code = left + ' = ' + 'paddle.nn.functional.' + act + '(' + left + ')'
            _create_simple_stmt_node_and_insert_behind(code, simple_stmt_node)
        # removed_value is a variable
        else:
            # create statement like "x = getattr(paddle.nn.functional, act)(x) if act else x"
            code = left + " = getattr(paddle.nn.functional, " + removed_value + ")(" + left + ") if " + removed_value + " else " + left
            _create_simple_stmt_node_and_insert_behind(code, simple_stmt_node)


def _create_simple_stmt_node_and_insert_behind(code, node):
    if node is None or node.type != python_symbols.simple_stmt:
        return
    simple_stmt_node = Node(python_symbols.simple_stmt, [utils.newline_node(node)])
    _node = utils.code_repr(code).children[0].children[0]
    _node.parent = None
    simple_stmt_node.insert_child(0, _node)
    simple_stmt_node.prefix = utils.get_indent(node)
    utils.insert_node_behind(node, simple_stmt_node)



import os
import sys
import shutil
from io import StringIO
from datetime import datetime
from fnmatch import filter

from fissix.pgen2 import driver
from fissix import pytree
from fissix.pygram import python_grammar, python_symbols
from fissix.pgen2 import token
from fissix.pytree import Leaf, Node
from fissix.fixer_util import Attr, Comma, Dot, LParen, Name, Newline, RParen, KeywordArg, Number, ArgList

from paddle1to2.common import logger

def log_debug(filename, lineno, msg):
    _msg = "{}:{} {}".format(filename, lineno, msg)
    logger.debug(_msg)

def log_info(filename, lineno, msg):
    _msg = "{}:{} {}".format(filename, lineno, msg)
    logger.info(_msg)

def log_warning(filename, lineno, msg):
    _msg = "{}:{} {}".format(filename, lineno, msg)
    logger.warning(_msg)

def log_error(filename, lineno, msg):
    _msg = "{}:{} {}".format(filename, lineno, msg)
    logger.error(_msg)

def node2code(nodes, with_first_prefix=False):
    """
    convert a node or a list of nodes to code str, e.g.
    "    import paddle" -> return "import paddle"
    "    import     paddle" -> return "import     paddle"
    """
    if not isinstance(nodes, list):
        nodes = [nodes]
    code = ''
    is_first_leaf = True
    for node in nodes:
        for leaf in node.leaves():
            if is_first_leaf:
                code = code + leaf.value
                is_first_leaf = False
            else:
                code = code + str(leaf)
    return code

def code_repr(src: str):
    """
    convert source code to node tree, e.g.
    src = 'path.to.api'
    output = "Node(power, [Leaf(1, 'path'), Node(trailer, [Leaf(23, '.'), Leaf(1, 'to')]), Node(trailer, [Leaf(23, '.'), Leaf(1, 'api')])])"
    """
    driver_ = driver.Driver(python_grammar, convert=pytree.convert)
    tree = driver_.parse_stream(StringIO(src + '\n'))
    return tree

def replace_module_path(node, before, after):
    """
    replace before with after in node.children, e.g.
    before = 'path1.to1.api1'
    after = 'path2.to2.api2'

    before replace, node is:
    Node(power, [Leaf(1, 'path1'), Node(trailer, [Leaf(23, '.'), Leaf(1, 'to1')]), Node(trailer, [Leaf(23, '.'), Leaf(1, 'api1')])])

    after replace, node is:
    Node(power, [Leaf(1, 'path2'), Node(trailer, [Leaf(23, '.'), Leaf(1, 'to2')]), Node(trailer, [Leaf(23, '.'), Leaf(1, 'api2')])])

    """
    before_parts = before.split('.')
    _node = code_repr(after).children[0].children[0]
    _node.parent = None
    after_node = _node
    # reserve prefix
    after_node.children[0].prefix = node.children[0].prefix
    del node.children[:len(before_parts)]
    for i in range(len(after_node.children)):
        node.insert_child(i, after_node.children[i])


def norm_arglist(trailer_node):
    """
    normilize argument list node, e.g.
    Node(trailer, [Leaf(7, '('), Leaf(2, '3'), Leaf(8, ')')])
    to
    Node(trailer, [Leaf(7, '('), Node(arglist, [Node(argument, [Leaf(2, '1')])]), Leaf(8, ')')])
    """
    if trailer_node.type != python_symbols.trailer or len(trailer_node.children) < 2 or len(trailer_node.children) > 3:
        logger.warning("node type is not trailer, or len(children) < 2 or len(children) > 3")
        return
    second_node = trailer_node.children[1]
    # add arglist node if needed
    if second_node.type != python_symbols.arglist:
        if second_node.type == token.RPAR:
            arglist_node = Node(python_symbols.arglist, [])
            trailer_node.insert_child(1, arglist_node)
        else:
            _second_node = second_node.clone()
            arglist_node = Node(python_symbols.arglist, [_second_node])
            second_node.remove()
            trailer_node.insert_child(1, arglist_node)
    else:
        arglist_node = second_node

    # add argument node if needed
    for i in range(len(arglist_node.children)):
        node = arglist_node.children[i]
        # skip "," node
        if node.type == token.COMMA:
            continue
        if node.type != python_symbols.argument:
            _node = node.clone()
            arg_node = Node(python_symbols.argument, [_node])
            node.replace(arg_node)

def add_argument(filename, trailer_node, key, value):
    """
    add "key=value" to arglist in trailer_node,
    if arglist already contains key, reassign key to value
    """
    if trailer_node.type != python_symbols.trailer and len(trailer_node.children) != 3:
        log_warning(filename, trailer_node.get_lineno(), "node type is not trailer or len(children) != 3. you may need to call norm_arglist first.")
        return
    arglist_node = trailer_node.children[1]
    if arglist_node.type != python_symbols.arglist:
        log_warning(filename, trailer_node.get_lineno(), "trailer_node.children[1] is not arglist.")
        return
    found_key = False
    for node in arglist_node.children:
        if node.type != python_symbols.argument:
            continue
        _key_node = node.children[0]
        if _key_node.value == key:
            found_key = True
            _value_node = node.children[2]
            if _value_node.value != value:
                _value_node_copy = _value_node.clone()
                _value_node_copy.type = token.NAME
                _value_node_copy.value = value
                _value_node.replace(_value_node_copy)
                log_warning(filename, trailer_node.get_lineno(), 'argument "{}" is reassigned to "{}"'.format(key, value))
            break
    if not found_key:
        key_node = Name(key)
        value_node = Name(value)
        if arglist_node.children:
            arglist_node.append_child(Comma())
            key_node.prefix = " "
        arg_node = KeywordArg(key_node, value_node)
        arglist_node.append_child(arg_node)
        log_warning(filename, trailer_node.get_lineno(), 'add argument "{}={}"'.format(key, value))


def remove_argument(filename, trailer_node, key):
    """
    remove "key" from arglist in trailer_node,
    """
    if trailer_node.type != python_symbols.trailer and len(trailer_node.children) != 3:
        log_warning(filename, trailer_node.get_lineno(), "node type is not trailer or len(children) != 3. you may need to call norm_arglist first.")
        return
    removed_value = None # record removed key=value and arglist_node
    arglist_node = trailer_node.children[1]
    if arglist_node.type != python_symbols.arglist:
        log_warning(filename, trailer_node.get_lineno(), "trailer_node.children[1] is not arglist.")
        return
    found_key = False
    for node in arglist_node.children:
        if node.type != python_symbols.argument:
            continue
        _key_node = node.children[0]
        if _key_node.value == key:
            found_key = True
            removed_value = node2code(node.children[2]) if len(node.children) >= 3 else None
            if node.prev_sibling is not None and node.prev_sibling.type is token.COMMA:
                node.prev_sibling.remove()
            elif node.next_sibling is not None and node.next_sibling.type is token.COMMA:
                node.next_sibling.remove()
            node.remove()
            log_warning(filename, trailer_node.get_lineno(), 'argument "{}" is removed.'.format(key))
            break
    if not found_key:
        log_info(filename, trailer_node.get_lineno(), 'argument "{}" not found.'.format(key))
    return removed_value


def rename_argument(filename, trailer_node, old_key, new_key):
    """
    rename "old_key" to "new_key" in arglist in trailer_node,
    """
    if trailer_node.type != python_symbols.trailer and len(trailer_node.children) != 3:
        log_warning(filename, trailer_node.get_lineno(), "node type is not trailer or len(children) != 3. you may need to call norm_arglist first.")
        return
    arglist_node = trailer_node.children[1]
    if arglist_node.type != python_symbols.arglist:
        log_warning(filename, trailer_node.get_lineno(), "trailer_node.children[1] is not arglist.")
        return
    found_key = False
    for node in arglist_node.children:
        if node.type != python_symbols.argument:
            continue
        _key_node = node.children[0]
        if _key_node.value == old_key:
            found_key = True
            _key_node.value = new_key
            log_warning(filename, trailer_node.get_lineno(), 'rename argument "{}" to "{}".'.format(old_key, new_key))
            break
    if not found_key:
        log_info(filename, trailer_node.get_lineno(), 'argument "{}" not found.'.format(old_key))


def apply_argument(filename, trailer_node, fun):
    """
    call fun(argument_node) for each argument
    """
    if trailer_node.type != python_symbols.trailer and len(trailer_node.children) != 3:
        log_warning(filename, trailer_node.get_lineno(), "node type is not trailer or len(children) != 3. you may need to call norm_arglist first.")
        return
    arglist_node = trailer_node.children[1]
    if arglist_node.type != python_symbols.arglist:
        log_warning(filename, trailer_node.get_lineno(), "trailer_node.children[1] is not arglist.")
        return
    for node in arglist_node.children:
        if node.type != python_symbols.argument:
            continue
        fun(node)


def startswith(module_path1, module_path2):
    """
    check module_path1 includes module_path2, e.g.
    "path.to.api.func_name" includes "path.to.api",
    but not include "path.api.func"

    return True if module_path1 includes module_path2, False otherwise.
    """
    # convert "path.to.api()" to "path.to.api"
    idx = module_path1.find('(')
    if idx != -1:
        module_path1 = module_path1[:idx]
    idx = module_path2.find('(')
    if idx != -1:
        module_path2 = module_path2[:idx]
    dotted_parts1 = module_path1.split('.')
    dotted_parts2 = module_path2.split('.')
    if len(dotted_parts1) < len(dotted_parts2):
        return False
    for i in range(len(dotted_parts2)):
        if dotted_parts1[i] != dotted_parts2[i]:
            return False
    return True

def _include_patterns(*patterns):
    """Factory function that can be used with copytree() ignore parameter.

    Arguments define a sequence of glob-style patterns
    that are used to specify what files to NOT ignore.
    Creates and returns a function that determines this for each directory
    in the file hierarchy rooted at the source directory when used with
    shutil.copytree().
    """
    def _ignore_patterns(path, names):
        keep = set(name for pattern in patterns
                            for name in filter(names, pattern))
        ignore = set(name for name in names
                        if name not in keep and not os.path.isdir(os.path.join(path, name)))
        return ignore
    return _ignore_patterns

def backup_inpath(inpath, backup):
    inpath = os.path.abspath(inpath)
    backup = os.path.abspath(backup)
    if not os.path.exists(backup):
        os.makedirs(backup)
    basename = os.path.basename(inpath)
    bak_basename = basename + "_backup_" + datetime.now().strftime("%Y_%m_%d_%H_%M_%S_%f")
    dst_path = os.path.join(backup, bak_basename)
    if os.path.isfile(inpath):
        shutil.copy(inpath, dst_path)
    else:
        shutil.copytree(inpath, dst_path, ignore=_include_patterns("*.py"))
    return dst_path

def dec_indent(indent, count=1):
    """
    decrease indent, e.g.
    if indent = "        ", and count = 1, return "    ",
    if indent = "        ", and count = 2, return "",
    """
    if indent.endswith('\t'):
        indent = indent[:len(indent) - 1 * count]
    elif indent.endswith('    '):
        indent = indent[:len(indent) - 4 * count]
    return indent

def get_indent(node):
    """
    get indent string of current node
    """
    while node is not None:
        if node.type == token.INDENT:
            return node.value
        node = node.prev_sibling
    return ""

def is_windows():
    """
    check if current operating system is windows
    """
    return sys.platform.lower() == 'win32'

def _is_windows_file(filepath):
    """
    if file contains "\r\n", treat it as windows file, and return True, else False
    """
    with open(filepath, 'rb') as f:
        content = f.read()
    if content.find(b'\r\n') != -1:
        return True
    return False

def _is_utf8(filepath):
    """
    check if file is utf8 encoding
    """
    with open(filepath, 'rb') as f:
        content = f.read()
    try:
        content_utf8 = content.decode('utf-8')
    except UnicodeDecodeError as e:
        return False
    return True

def valid_path(inpath):
    # check if inpath is valid
    if not os.path.exists(inpath):
        logger.error("{} doesn't exist.".format(inpath))
        return False
    valid = True
    if os.path.isfile(inpath):
        """
        refactor windows files on linux or mac os is not supported.
        """
        if not is_windows():
            if _is_windows_file(inpath):
                logger.error('{} is a windows file, you can use "dos2unix" command to convert it to linux file.'.format(inpath))
                valid = False
            if not _is_utf8(inpath):
                logger.error('{} encoding is not utf-8, you can use "iconv" command to convert it to utf-8.'.format(inpath))
                valid = False
    elif os.path.isdir(inpath):
        for dirpath, dirnames, filenames in os.walk(inpath):
            for filename in filenames:
                if not filename.endswith('.py'):
                    continue
                filepath = os.path.join(dirpath, filename)
                if not is_windows() and _is_windows_file(filepath):
                    logger.error('{} is a windows file, you can use "dos2unix" command to convert it to linux file.'.format(filepath))
                    valid = False
                if not _is_utf8(filepath):
                    logger.error('{} encoding is not utf-8, you can use "iconv" command to convert it to utf-8.'.format(filepath))
                    valid = False
    else:
        logger.error('{} is neither a file nor a directory.'.format(inpath))
        valid = False

    return valid

def get_node_index(node):
    """
    get index of node in its parent's children
    """
    if node is None or node.parent is None:
        return -1
    for idx, child in enumerate(node.parent.children):
        if child is node:
            return idx
    return -1

def insert_node_behind(node1, nodes):
    """
    insert nodes behind node1
    """
    if node1 is None or node1.parent is None:
        return
    idx = get_node_index(node1)
    if idx != -1:
        idx += 1
        if not isinstance(nodes, list):
            nodes = [nodes,]
            for node in nodes:
                if node is not None:
                    node1.parent.insert_child(idx, node)
                    idx += 1

_newline_node = None
def newline_node(node):
    """
    Find a NEWLINE node from AST tree, and clone it to create a new NEWLINE node.
    So we don't need to care about os platform.
    if no NEWLINE node found in AST tree, create a new NEWLINE node according to os platform.
    """
    global _newline_node
    if _newline_node is None:
        p = node
        while p is not None and p.parent is not None:
            p = p.parent
        if p is not None:
            for leaf in p.leaves():
                if leaf.type == token.NEWLINE:
                    _newline_node = leaf
                    break
        if _newline_node is None:
            if is_windows:
                _newline_node = Newline(value="\r\n")
            else:
                _newline_node = Newline()
    return _newline_node.clone()

def is_import_node(node):
    """
    check if a node is an import node
    """
    p = node
    while p is not None:
        if p.type in {python_symbols.import_name, python_symbols.import_from}:
            return True
        p = p.parent
    return False

def is_argument_node(node):
    """
    check if a node is a argument node
    """
    p = node
    while p is not None:
        if p.type == python_symbols.argument:
            return True
        p = p.parent
    return False

def is_left_operand(node):
    """
    check if a node is left operand
    """
    if node is not None and node.next_sibling is not None and node.next_sibling.type is token.EQUAL:
        return True
    return False



# ref: https://github.com/pallets/click/
import sys

_ansi_colors = {
    "black": 30,
    "red": 31,
    "green": 32,
    "yellow": 33,
    "blue": 34,
    "magenta": 35,
    "cyan": 36,
    "white": 37,
    "reset": 39,
    "bright_black": 90,
    "bright_red": 91,
    "bright_green": 92,
    "bright_yellow": 93,
    "bright_blue": 94,
    "bright_magenta": 95,
    "bright_cyan": 96,
    "bright_white": 97,
}

_ansi_reset_all = "\033[0m"

def _interpret_color(color, offset=0):
    if isinstance(color, int):
        return "{};5;{}".format(38 + offset, color)

    if isinstance(color, (tuple, list)):
        r, g, b = color
        return "{};2;{};{};{}".format(38 + offset, r, g, b)

    return str(_ansi_colors[color] + offset)

def style(
    text,
    fg=None,
    bg=None,
    bold=None,
    dim=None,
    underline=None,
    blink=None,
    reverse=None,
    reset=True,
):
    if not isinstance(text, str):
        text = str(text)

    if sys.platform.lower() == 'win32':
        return text

    bits = []

    if fg:
        try:
            bits.append("\033[0;{}m".format(_interpret_color(fg)))
        except KeyError:
            raise TypeError("Unknown color {}".format(fg))

    if bg:
        try:
            bits.append("\033[0;{}m".format(_interpret_color(bg, 10)))
        except KeyError:
            raise TypeError("Unknown color {}".format(fg))

    if bold is not None:
        bits.append("\033[{}m".format(1 if bold else 22))
    if dim is not None:
        bits.append("\033[{}m".format(2 if dim else 22))
    if underline is not None:
        bits.append("\033[{}m".format(4 if underline else 24))
    if blink is not None:
        bits.append("\033[{}m".format(5 if blink else 25))
    if reverse is not None:
        bits.append("\033[{}m".format(7 if reverse else 27))
    bits.append(text)
    if reset:
        bits.append(_ansi_reset_all)
    return "".join(bits)

def secho(message=None, nl=True, err=False, **styles):
    if message is not None:
        message = style(message, **styles)

    return echo(message, nl=nl, err=err)

def echo(message=None, nl=True, err=False):
    if err:
        out = sys.stderr
    else:
        out = sys.stdout

    # Convert non bytes/text into the native string type.
    if message is not None and not isinstance(message, str):
        message = str(message)

    if nl:
        message = message or ""
        if isinstance(message, str):
            message += "\n"
        else:
            message += b"\n"

    if message:
        out.write(message)
    out.flush()

class Abort(RuntimeError):
    """An internal signalling exception that signals Click to abort."""

def confirm(text, default=False, abort=False, prompt_suffix=": ", err=False):
    prompt = "{} [{}]{}".format(text, "Y/n", prompt_suffix)
    while 1:
        try:
            # Write the prompt separately so that we get nice
            # coloring through colorama on Windows
            echo(prompt, nl=False, err=err)
            value = input("").lower().strip()
        except (KeyboardInterrupt, EOFError):
            raise Abort()
        if value in ("y", "yes"):
            rv = True
        elif value in ("n", "no"):
            rv = False
        elif value == "":
            rv = default
        else:
            echo("Error: invalid input", err=err)
            continue
        break
    if abort and not rv:
        raise Abort()
    return rv
